#include "main.h"
//==============================================================
static const char *TAG = "main";
//==============================================================
static void ir_rx_task(void *arg)
{
  size_t length = 0;
  uint32_t addr = 0;
  uint32_t cmd = 0;
  RingbufHandle_t rb = NULL;
  rmt_item32_t *items = NULL;
  uint8_t lvl = 1;
  bool repeat = false;
  rmt_config_t rmt_rx_config = RMT_DEFAULT_CONFIG_RX(CONFIG_RMT_RX_GPIO, CONFIG_RMT_RX_CHANNEL);
  rmt_config(&rmt_rx_config);
  rmt_driver_install(CONFIG_RMT_RX_CHANNEL, 1000, 0);
  ir_parser_config_t ir_parser_config = IR_PARSER_DEFAULT_CONFIG((ir_dev_t)CONFIG_RMT_RX_CHANNEL);
  ir_parser_config.flags |= IR_TOOLS_FLAGS_PROTO_EXT; // Using extended IR protocols (both NEC and RC5 have extended version)
  ir_parser_t *ir_parser = NULL;
#if CONFIG_IR_PROTOCOL_NEC
  ir_parser = ir_parser_rmt_new_nec(&ir_parser_config);
#elif CONFIG_IR_PROTOCOL_RC5
  ir_parser = ir_parser_rmt_new_rc5(&ir_parser_config);
#endif
  rmt_get_ringbuf_handle(CONFIG_RMT_RX_CHANNEL, &rb);
  assert(rb != NULL);
  rmt_rx_start(CONFIG_RMT_RX_CHANNEL, true);
  while (1)
  {
    items = (rmt_item32_t *) xRingbufferReceive(rb, &length, portMAX_DELAY);
    if (items)
    {
      gpio_set_level(CONFIG_BLINK_GPIO, lvl);
      lvl ^= 1;
      length /= 4; // one RMT = 4 Bytes
      if (ir_parser->input(ir_parser, items, length) == ESP_OK)
      {
        if (ir_parser->get_scan_code(ir_parser, &addr, &cmd, &repeat) == ESP_OK)
        {
#if CONFIG_IR_PROTOCOL_NEC
          if(((uint8_t)addr == (uint8_t)~(addr>>8)) && ((uint8_t)cmd == (uint8_t)~(cmd>>8)))
          {
              ESP_LOGI(TAG, "Scan Code %s --- addr: 0x%02x, cmd: 0x%02x", repeat ? "(repeat)" : "",
                    (uint8_t)addr, (uint8_t)cmd);
          }
          else
          {
            ESP_LOGE(TAG, "Scan Code XOR Error!!!");
            ESP_LOGI(TAG, "Scan Code %s --- addr: 0x%04x cmd: 0x%04x", repeat ? "(repeat)" : "", addr, cmd);
          }
#elif CONFIG_IR_PROTOCOL_RC5
          ESP_LOGI(TAG, "Scan Code %s --- addr: 0x%04x cmd: 0x%04x", repeat ? "(repeat)" : "", addr, cmd);
#endif
        }
      }
      vRingbufferReturnItem(rb, (void *) items);
    }
  }
  ir_parser->del(ir_parser);
  rmt_driver_uninstall(CONFIG_RMT_RX_CHANNEL);
  vTaskDelete(NULL);
}
//==============================================================
void app_main(void)
{
  gpio_reset_pin(CONFIG_BLINK_GPIO);
  gpio_set_direction(CONFIG_BLINK_GPIO, GPIO_MODE_OUTPUT);
  gpio_set_level(CONFIG_BLINK_GPIO, 0);
  xTaskCreate(ir_rx_task, "ir_rx_task", 2048, NULL, 10, NULL);
}
//==============================================================
